/*******************************************************************************
NAME:                            TESTPROJ 

PURPOSE:	Makes a grid of GCTPc converted map projection
		points.  The User enters the Min and Max Lat and Long
		and the increment value.  A grid of points will be converted
		from Long/Lat to X/Y and back to Long/Lat.  The results
		will be entered into a file specified by the user.  The
		user optionally may compare the transformed data to the data
		in an existing file of the same format.


ALGORITHM REFERENCES

1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Proffessional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "cproj.h"
#include "proj.h"
#include "gctp.h"

#define CMLEN	256

int main(void)
{
    FILE *fptr1 = NULL;
    FILE *fptr2 = NULL;
    FILE *fptr3 = NULL;

    char buf2[82];	/* buffer for file2 information			*/
    char efile[CMLEN];	/* name of error file				*/
    char fflag[2];	/* flag for saving converted values to a file	*/
    char file1[CMLEN];	/* name of new data file     			*/
    char file2[CMLEN];	/* name of existing file			*/
    char file3[CMLEN];	/* name of Comparison file			*/
    char temp2[82];	/* temporary buffer for header strings		*/

    double cmp_incoor1[2];
    double cmp_incoor2[2];
    double cmp_outcoor1[2];
    double false_easting;
    double incoor1[2];
    double incoor2[2];
    double outcoor1[2];
    double outcoor2[2];

    GCTP_PROJECTION geo_proj;   /* geographic projection info */
    GCTP_PROJECTION test_proj;  /* projection being tested */
    GCTP_TRANSFORMATION *to_latlon = NULL;
    GCTP_TRANSFORMATION *to_meters = NULL;

    double i;
    double j;
    double lat_inc;
    double lon_inc;
    double max_lat;
    double max_lon;
    double min_lat;
    double min_lon;
    double tol;		/* tolerance value for acceptable comparisons	*/

    int count;
    int k;

    long count2;

    memset(&geo_proj, 0, sizeof(geo_proj));
    memset(&test_proj, 0, sizeof(test_proj));

    geo_proj.proj_code = 0;
    geo_proj.units = DEGREE;
    geo_proj.zone = 62;

    test_proj.spheroid = -2;
    test_proj.units = METER;
    test_proj.zone = 62;

    /* Assign file names
       -----------------*/
    strcpy (efile,"error_file.txt");

    /* Get needed information
    -----------------------*/
    printf(" 0    GEOGRAPHIC		");
    printf(" 1    UTM \n");
    printf(" 2    STATE PLANE		");
    printf(" 3    ALBERS CONICAL EQ AR \n");
    printf(" 4    LAMBERT CONFORMAL		");
    printf(" 5    MERCATOR \n");
    printf(" 6    POLAR STEREOGRAPHIC	");
    printf(" 7    POLYCONIC \n");
    printf(" 8    EQUIDISTANT CONIC		");
    printf(" 9    TRANSVERSE MERCATOR \n");
    printf("10    STEREOGRAPHIC		");
    printf("11    LAMBERT AZIMUTHAL \n");
    printf("12    AZIMUTHAL EQUDISTANT	");
    printf("13    GNOMONIC \n");
    printf("14    ORHTOGRAPHIC		");
    printf("15    GVNSP \n");
    printf("16    SINUSIODAL		");
    printf("17    EQUIRECTANGULAR \n");
    printf("18    MILLER			");
    printf("19    VAN DER GRINTEN \n");
    printf("20    OBLIQUE MERCATOR		");
    printf("21    ROBINSON \n");
    printf("22    SOM			");
    printf("23    ALASKA CONFORMAL \n");
    printf("24    GOODE			");
    printf("25    MOLLWEIDE \n");
    printf("26    INTERRUPTED MOLLEIDE	");
    printf("27    HAMMER \n");
    printf("28    WAGNER IV			");
    printf("29    WAGNER VII \n");
    printf("30    OBLATED EQ AR		");
    printf("31    INTEGERIZED SINUSOIDAL\n");

    printf(" Please enter the projection number \n");
    scanf("%d",&test_proj.proj_code);

    if ((test_proj.proj_code == UTM) || (test_proj.proj_code == SPCS))
    {
        printf("\n Please enter the zone number \n");
        scanf("%d",&test_proj.zone);

        printf(" 0    CLARKE 1866             ");
        printf(" 1    CLARKE 1880 \n");
        printf(" 2    BESSEL                  ");
        printf(" 3    INTERNATIONAL 1967 \n");
        printf(" 4    INTERNATIONAL 1909      ");
        printf(" 5    WGS 72 \n");
        printf(" 6    EVEREST                 ");
        printf(" 7    WGS 66 \n");
        printf(" 8    GRS 1980 (NAD 1983)     ");
        printf(" 9    AIRY \n");
        printf("10    MODIFIED EVEREST        ");
        printf("11    MODIFIED AIRY \n");
        printf("12    WGS 84                  ");
        printf("13    SOUTHEAST ASIA \n");
        printf("14    AUSTRALIAN NATIONAL     ");
        printf("15    KRASOVSKY \n");
        printf("16    HOUGH                   ");
        printf("17    MERCURY 1960 \n");
        printf("18    MODIFIED MERCURY 1968   ");
        printf("19    6370997 SPHERE \n");
        printf("20    BESSEL 1841             ");
        printf("21    EVEREST SABAH & SARAWAK \n");
        printf("22    EVEREST INDIA 1956      ");
        printf("23    EVEREST MALAYSIA 1969 \n");
        printf("24    EVEREST MALAY/SING 1948 ");
        printf("25    EVEREST PAKISTAN \n");
        printf("26    HAYFORD                 ");
        printf("27    HELMERT 1906 \n");
        printf("28    INDONESIAN 1974         ");
        printf("29    SOUTH AMERICAN 1969 \n");
        printf("30    WGS 60                  ");
        printf("31    MODIS SPHERE\n");

        printf(" Please enter the spheroid/datum number \n");
        scanf("%d",&test_proj.spheroid);
    }
    else
    {
        test_proj.zone = 62;
    }

    for (k = 0; k < 15; k++)
    {
        printf("\n Please enter projection parameter %d \n",k);
        scanf("%lf",&test_proj.parameters[k]);
        geo_proj.parameters[k] = 0.0;
    }

    printf("\nPlease enter the minimum longitude in degrees (negative "
           "for west of zero)\n");
    scanf("%lf",&min_lon);
    printf("\nPlease enter the maximum longitude in degrees (negative "
           "for west of zero)\n");
    scanf("%lf",&max_lon);
    printf("\n Please enter the minimum latitude in degrees\n");
    scanf("%lf",&min_lat);
    printf("\n Please enter the maximum latitude in degrees\n");
    scanf("%lf",&max_lat);

    printf("\n Please enter the longitude increment in degrees\n");
    scanf("%lf",&lon_inc);
    printf("\n Please enter the latitude increment in degrees\n");
    scanf("%lf",&lat_inc);

    /* Get name of new data file
       -------------------------*/
    printf("\n Please enter name of the new data file\n");
    scanf("%s",file1);

    /* If files are to be saved with converted values get file names
      -------------------------------------------------------------*/
    printf("\n Are the values to be compared? (Y or N)\n");
    scanf("%s",fflag);
    *fflag = toupper(*fflag);

    if (!strcmp(fflag,"Y"))
    {
        printf("\n Please enter name of the existing data file\n");
        scanf("%s",file2);
    }

    if (!strcmp(fflag,"Y"))
    {
        printf("\n Please enter the tolerance value\n");
        scanf("%lf",&tol);

        printf("\n Please enter name of the Comparison file\n");
        scanf("%s",file3);
        fptr3 = fopen(file3,"w");
        if (fptr3 == NULL)
        {
            printf("\nError opening output comparison file %s\n",file3);
            return 1;
        }

        /* open existing data file
          ----------------------*/
        fptr2 = fopen(file2,"r");
        if (fptr2 == NULL)
        {
            printf("\nError opening comparison file--discontinue comparison\n");
            fprintf(fptr3,"\nError opening comp file--discontinue "
                    "comparison\n");
            strcpy(fflag,"N");
            fclose(fptr3);
        }
        count2 = 0;

        /* Get past the heading, will stop after the line beginning with "-"
          ----------------------------------------------------------------*/
        if (!strcmp(fflag,"Y"))
        {
            int status;

            for(;;)
            {
                count2++;
                if (fgets(buf2,81,fptr2) == NULL)
                {
                    printf("\nError reading comparison file--discontinue "
                           "comparison\n");
                    fprintf(fptr3,"\nError reading comp file--discontinue "
                            "comparison\n");
                    strcpy(fflag,"N");
                    fclose(fptr2);
                    fclose(fptr3);
                    break;
                }

                /* Break at "-",  test to avoid invalid data file (count2)
                ------------------------------------------------------*/
                status = sscanf(buf2,"%s",temp2);
                if ((status >= 1) && (strlen(temp2) >= 2) && temp2[1] == '-')
                    break;
                else if (count2 > 50)
                {
                    printf("\nError reading comparison file--discontinue "
                           "comparison\n");
                    fprintf(fptr3,"\nError reading comp file--discontinue "
                            "comparison\n");
                    strcpy(fflag,"N");
                    fclose(fptr2);
                    fclose(fptr3);
                    break;
                }
            }
        }
    }

    /* Set the false easting from the projection parameter at index 6, except 
       for projections geo, utm, spcs, good, imoll for which it is undefined */
    if ((test_proj.proj_code == GEO) || (test_proj.proj_code == UTM) || (test_proj.proj_code == SPCS) ||
        (test_proj.proj_code == GOOD) || (test_proj.proj_code == IMOLL))
    {
        false_easting = 0.0;
    }
    else
        false_easting = test_proj.parameters[6];

    /* Empty contents of new data file if it exists
    ---------------------------------------------*/
    if((fptr1 = fopen(file1,"w")) != NULL)
        fclose(fptr1);

    /* Test projection 
    ----------------*/
    /* set variables not entered with prompt 
       ------------------------------------*/
    count = 0;

     /*  Set values to change the datum and radius
     --------------------------------------------*/
     geo_proj.spheroid = 0;
     if ((test_proj.parameters[0] != 0) && (test_proj.parameters[0] != 6370997))
         if ((test_proj.proj_code != UTM) && (test_proj.proj_code != SPCS))
             geo_proj.spheroid = -1;
     if(test_proj.spheroid == -2)  /* so we don't lose the datum input for UTM & SPCS */
        test_proj.spheroid = geo_proj.spheroid;


    /* Create the transformations */
    to_latlon = gctp_create_transformation(&test_proj, &geo_proj);
    if (!to_latlon)
    {
        printf("Error creating projection to lat/lon\n");
        return 1;
    }
    gctp_print_transformation_info(to_latlon);

    to_meters = gctp_create_transformation(&geo_proj, &test_proj);
    if (!to_meters)
    {
        printf("Error creating projection to meters\n");
        gctp_destroy_transformation(to_latlon);
        return 1;
    }
    gctp_print_transformation_info(to_meters);

    /*  Begin convertion of grid points
       -------------------------------*/
    for(j = min_lon; j <= max_lon; j += fabs(lon_inc))
    {
        for(i = min_lat; i <= max_lat; i += fabs(lat_inc))
        {
            incoor1[1] =  i;
            incoor1[0] =  j;

            /* Call "C" version forward--lat/long to meters
            --------------------------------------------*/
            if (gctp_transform(to_meters, incoor1, outcoor1) != GCTP_SUCCESS)
            {
                printf("Error converting to meters\n");
                gctp_destroy_transformation(to_latlon);
                gctp_destroy_transformation(to_meters);
                return 1;
            }

            /* Print Headings
            -------------*/
            if ((i == min_lat) && (j == min_lon))
            {
                if((fptr1 = fopen(file1,"a")) == NULL)
                {
                    printf("\nError opening output data file");
                    gctp_destroy_transformation(to_latlon);
                    gctp_destroy_transformation(to_meters);
                    return 1;
                }
                fprintf(fptr1,"\n INPUT LONG LAT        X              Y   "
                        "        OUTPUT LON      OUTPUT LAT");
                fprintf(fptr1,"\n--------------- --------------- "
                        "--------------- --------------- ---------------\n");
                fclose(fptr1);
            }

            outcoor2[0] = outcoor1[0];
            outcoor2[1] = outcoor1[1];

            /* Call "C" version inverse--meters to lat/long
            --------------------------------------------*/
            if (gctp_transform(to_latlon, outcoor2, incoor2) != GCTP_SUCCESS)
            {
                printf("Error converting to lat/lon\n");
                gctp_destroy_transformation(to_latlon);
                gctp_destroy_transformation(to_meters);
                return 1;
            }

            /*  Print points to file
            -------------------*/
            if((fptr1 = fopen(file1,"a")) == NULL)
            {
                printf("\nError opening output data file");
                gctp_destroy_transformation(to_latlon);
                gctp_destroy_transformation(to_meters);
                return 1;
            }
            fprintf(fptr1,"%7.2f %6.2f %15.5f %15.5f %15.5f %15.5f\n",
                    incoor1[0],incoor1[1],outcoor1[0],outcoor1[1],incoor2[0],
                    incoor2[1]);
            fclose(fptr1);

            count++;

            /* Compare values to data file
            -----------------------------*/
            if (!strcmp(fflag,"Y"))
            {
                if (fscanf(fptr2,"%lf %lf %lf %lf %lf %lf",&cmp_incoor1[0],
                           &cmp_incoor1[1],&cmp_outcoor1[0],&cmp_outcoor1[1],
                           &cmp_incoor2[0],&cmp_incoor2[1]) != 6)
                {
                    printf("\nError reading comparison file--discontinue "
                           "comparison\n");
                    fprintf(fptr3,"\nError reading comp file--discontinue "
                            "comparison\n");
                    strcpy(fflag,"N");
                    fclose(fptr3);
                    fclose(fptr2);
                }

                /* Check if original lat/long are the same--if not, error in 
                 * comp file
                -------------------------------------------------------------*/
                if (!strcmp(fflag,"Y"))
                {
                    if(fabs(incoor1[0]-cmp_incoor1[0])>tol || 
                       (fabs(incoor1[1]-cmp_incoor1[1])>tol))
                    {
                        printf("\nInconsistant input Long/Lat--discontinue "
                               "comparison\n");
                        fprintf(fptr3,"\nInconsistant input Long/Lat--"
                                "discontinue comparison\n");
                        strcpy(fflag,"N");
                        fclose(fptr3);
                        fclose(fptr2);
                    }
                }
                if (!strcmp(fflag,"Y"))
                {
                    /* if the difference (including tolerance is greater than 
                       360, convert the negative value to positive to make sure
                       -180 vs +180 isn't reported as a difference */
                    if (fabs(incoor2[0] - cmp_incoor2[0]) + tol > 360.0)
                    {
                        if (incoor2[0] < 0.0)
                        incoor2[0] += 360.0;
                        if (cmp_incoor2[0] < 0.0)
                        cmp_incoor2[0] += 360.0;
                    }

                    /* if the input latitude is +/- 90 (within tolerance) and 
                       X adjusted for the false easting is zero, ignore the 
                       comparison on the longitudes since it degenerates to a 
                       single point */
                    if ((fabs((fabs(cmp_incoor1[1]) - 90.0)) < tol) && 
                        (fabs(cmp_outcoor1[0] - false_easting) < tol))
                    {
                        incoor2[0] = cmp_incoor2[0];
                    }

                    if ((fabs(outcoor1[0] - cmp_outcoor1[0]) > tol) ||
                        (fabs(outcoor1[1] - cmp_outcoor1[1]) > tol))
                    {
                        fprintf(fptr3,"Differences exist at long %f lat %f\n",
                                incoor1[0],incoor1[1]);
                        printf("\nDifferences exist at long %f lat %f",
                                incoor1[0],incoor1[1]);
                        fprintf(fptr3," new X %17.6f Y %17.6f\n"
                                      " old X %17.6f Y %17.6f\n",
                                outcoor1[0],outcoor1[1],cmp_outcoor1[0],
                                cmp_outcoor1[1]);
                    }
                    else if ((fabs(incoor2[0] - cmp_incoor2[0]) > tol) ||
                             (fabs(incoor2[1] - cmp_incoor2[1]) > tol))
                    {
                        fprintf(fptr3,"Differences exist at long %f lat %f\n",
                                incoor1[0],incoor1[1]);
                        printf("\nDifferences exist at long %f lat %f",
                                incoor1[0],incoor1[1]);
                        fprintf(fptr3,"lon new %f lon old %f lat new %f lat "
                                "old %f\n",incoor2[0],cmp_incoor2[0],incoor2[1],
                                cmp_incoor2[1]);
                    }
                }
            }			/* end if comparison checks */
        }			/* end of for i */
    }			/* end of for j */
    if (!strcmp(fflag,"Y"))
    {
        fclose(fptr3);
        fclose(fptr2);
    }
    printf("\nNumber of points transformed %d \n",count);

    gctp_destroy_transformation(to_latlon);
    gctp_destroy_transformation(to_meters);

    return 0;   

}
